/* proto.h - The prototypes for the dbm routines. */

/*  This file is part of GDBM, the GNU data base manager, by Philip A. Nelson.
    Copyright (C) 1990, 1991, 1993  Free Software Foundation, Inc.

    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.

    GDBM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GDBM; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@cs.wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226

*************************************************************************/

/* PATCHED VERSION OF GDBM LIBRARY *******************************************
 * This patch is intended to copy the functionality of the 
 * GDBM library as it is found on the SGI platform.
 * The original version of the library created different 
 * fileformats at different platforms. This patched version
 * intends to create the same file at each different platform.
 * The current patched version has been tested for OS X, IRIX,
 * and LINUX. As the original files were all created under
 * IRIX, this patch conforms to the files created at that
 * platform.
 *
 * patch includes:
 * 1) definition of off_t as my_int64_t to be sure that this
 *    field is 8 bytes across platforms. off_t is 4 bytes
 *    on linux and 8 bytes on sgi and osx.
 * 2) includes byteswapping to store the files in the SGI
 *    native format to ensure cross platform compatibility.
 * 3) saving data structures field by field to ensure that
 *    no zero padding is added in between members of these
 *    structures.
 * 4) to add empty dummy fields at those places where the original
 *    SGI version of this library did put additional zeros to have
 *    64 bit integers align properly in memory.
 *
 * notes:
 * the definition of my_int64_t in autoconf.h may need some adjustments
 * in the future to be compatible with more compilers.
 *
 * All lines that have been patched are marked with a PATCH marker
 * the only exception is the conversion of off_t to my_int64_t
 *
 * patch generated by : JG Stinstra
 *
 ****************************************************************************/

/* Update version */


#ifndef __P
#ifdef __STDC__
#define __P(args) args
#else
#define __P(args) ()
#endif
#endif


/* From bucket.c */
void _gdbm_new_bucket    __P((gdbm_file_info *, hash_bucket *, int));
void _gdbm_get_bucket    __P((gdbm_file_info *, int));
void _gdbm_split_bucket  __P((gdbm_file_info *, int));
void _gdbm_write_bucket  __P((gdbm_file_info *, cache_elem *));

/* From falloc.c */
my_int64_t _gdbm_alloc       __P((gdbm_file_info *, int));
void _gdbm_free         __P((gdbm_file_info *, my_int64_t, int));
int  _gdbm_put_av_elem  __P((avail_elem, avail_elem [], int *, int));

/* From findkey.c */
char *_gdbm_read_entry  __P((gdbm_file_info *, int));
int _gdbm_findkey       __P((gdbm_file_info *, datum, char **, int *));

/* From hash.c */
int _gdbm_hash __P((datum));

/* From update.c */
void _gdbm_end_update   __P((gdbm_file_info *));
int _gdbm_fatal         __P((gdbm_file_info *, char *));

/* From gdbmopen.c */
int _gdbm_init_cache	__P((gdbm_file_info *, int));

/* The user callable routines.... */
void  gdbm_close	  __P((gdbm_file_info *));
int   gdbm_delete	  __P((gdbm_file_info *, datum));
datum gdbm_fetch	  __P((gdbm_file_info *, datum));
gdbm_file_info *gdbm_open __P((char *, int, int, int, void (*) (void)));
int   gdbm_reorganize	  __P((gdbm_file_info *));
datum gdbm_firstkey       __P((gdbm_file_info *));
datum gdbm_nextkey        __P((gdbm_file_info *, datum));
int   gdbm_store          __P((gdbm_file_info *, datum, datum, int));
int   gdbm_exists	  __P((gdbm_file_info *, datum));
void  gdbm_sync		  __P((gdbm_file_info *));
int   gdbm_setopt	  __P((gdbm_file_info *, int, int *, int));
int   gdbm_fdesc	  __P((gdbm_file_info *));


